using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Xml.Serialization;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace EQIA.EQIDL
{
    // TODO : Fix the issue with animations that actually animate, I haven't really checked into what's causing
    //  the problem yet so some exploration will be required, but my suspicion is that the IDrawableSurface
    //  implimentation is still kind of weak.

    /// <summary>
    /// A collection of frames that make up an animation.
    /// </summary>
    public partial class Ui2DAnimation : Class, IReferer, ILoadable, IUpdateable, IDrawable, IDrawableSurface
    {
        #region Members

        List<Frame> myFrames;
        private int myCellWidth, myCellHeight;
        private bool myCycle, myGrid, myVertical;

        #endregion

        #region Properties

        [Browsable(true), Description("Cycle the animation")]
        [XmlElement("Cycle", IsNullable = false)]
        public bool Cycle { get { return myCycle; } set { myCycle = value; } }

        [Browsable(true), Description("Set this animation to be a grid (used for drag items, etc.)")]
        [XmlElement("Grid", IsNullable = false)]
        public bool Grid { get { return myGrid; } set { myGrid = value; } }

        [Browsable(true), Description("Grid is Vertical instead of horizontal (only used when Grid = true)")]
        [XmlElement("Vertical", IsNullable = false)]
        public bool Vertical { get { return myVertical; } set { myVertical = value; } }

        [Browsable(true), Description("Width of each cell in the grid (only used when Grid = true)")]
        [XmlElement("CellWidth", IsNullable = false)]
        public int CellWidth { get { return myCellWidth; } set { myCellWidth = value; } }

        [Browsable(true), Description("Height of each cell in the grid (only used when Grid = true)")]
        [XmlElement("CellHeight", IsNullable = false)]
        public int CellHeight { get { return myCellHeight; } set { myCellHeight = value; } }

        [Browsable(true), Description("Animation frames")]
        [XmlElement("Frames", IsNullable = false)]
        public List<Frame> Frames { get { return myFrames; } set { myFrames = value; } }

        #endregion

        #region Creation

        public Ui2DAnimation()
            : this(null)
        {
        }

        public Ui2DAnimation(EQInterface eqInterface)
            : base(eqInterface)
        {
        }

        #endregion

        #region Deserialization

        protected override void SetProperties(object obj)
        {
            base.SetProperties(obj);

            Ui2DAnimation temp_Ui2DAnimation = obj as Ui2DAnimation;
            if (obj != null)
            {
                Cycle = temp_Ui2DAnimation.Cycle;
                Grid = temp_Ui2DAnimation.Grid;
                Vertical = temp_Ui2DAnimation.Vertical;
                CellWidth = temp_Ui2DAnimation.CellWidth;
                CellHeight = temp_Ui2DAnimation.CellHeight;
                Frames = temp_Ui2DAnimation.Frames;
            }
        }

        #endregion

        #region IReferer Members

        public void ResolveReferences()
        {
            for (int i = 0; myFrames != null && i < myFrames.Count; i++)
            {
                myFrames[i].Interface = Interface;
                myFrames[i].ResolveReferences();
            }
        }

        public bool ValidateReferences()
        {
            for (int i = 0; myFrames != null && i < myFrames.Count; i++)
                if (!myFrames[i].ValidateReferences()) 
                    return false;
            
            return true;
        }

        #endregion

        #region ILoadable Members

        [XmlIgnore()]
#if DEBUG
        [Browsable(true), Category("Internal Data"), NotifyParentProperty(true)]
#else
        [Browsable(false)]
#endif
        public bool IsLoaded
        {
            get
            {
                for (int i = 0; myFrames != null && i < myFrames.Count; i++)
                    if (!myFrames[i].IsLoaded) return false;

                return true;
            }
        }

        public void LoadContent(GraphicsDevice device)
        {
            for (int i = 0; myFrames != null && i < myFrames.Count; i++)
                myFrames[i].LoadContent(device);
        }

        public void UnloadContent()
        {
            for (int i = 0; myFrames != null && i < myFrames.Count; i++)
                myFrames[i].UnloadContent();
        }

        #endregion

        #region IUpdateable Members

        private int myCurrentFrame = 0;
        private int myElapsedTime = 0;

        public void Update(GameTime gameTime)
        {
            // Check if there are frames in this animation
            if (myFrames != null && myFrames.Count > 1)
            {
                // Add the elapsed miliseconds to the counter
                myElapsedTime += gameTime.ElapsedGameTime.Milliseconds;

                // If the counter is bigger than the duration of the current frame, or equal
                if (myElapsedTime >= myFrames[myCurrentFrame].Duration)
                {
                    // Reset the counter
                    myElapsedTime = 0;

                    int pFrame = myCurrentFrame;
                    myCurrentFrame = (myCycle && myCurrentFrame + 1 < myFrames.Count) ?
                        myCurrentFrame + 1 :
                        0;

                    if (pFrame != myCurrentFrame)
                        isRequireRedraw = true;
                }
            }
        }

        #endregion

        #region IDrawable Members

        public void Draw(SpriteBatch batch) { Draw(batch, Point.Zero, Size.Empty); }

        public void Draw(SpriteBatch batch, Point location) { Draw(batch, location, Size.Empty); }

        public void Draw(SpriteBatch batch, Point location, Size size)
        {
            if (myFrames != null && myCurrentFrame < myFrames.Count)
                myFrames[myCurrentFrame].Draw(batch, location, size);
        }

        #endregion

        #region IDrawableSurface Members

        private bool isRequireRedraw;

        [XmlIgnore()]
#if DEBUG
        [Browsable(true), Category("Internal Data"), NotifyParentProperty(true)]
#else
        [Browsable(false)]
#endif
        public bool IsRequireRedraw
        {
            get 
            {
                if (!isRequireRedraw)
                {
                    for (int i = 0; myFrames != null && i < myFrames.Count; i++)
                        if (myFrames[i].IsRequireRedraw) return true;
                }
                return false;
            }
        }

        public void DrawToTexture(SpriteBatch batch)
        {
            for (int i = 0; myFrames != null && i < myFrames.Count; i++)
                if (myFrames[i].IsRequireRedraw) myFrames[i].DrawToTexture(batch);
            
            isRequireRedraw = false;
        }

        public void EnsureRenderTarget(GraphicsDevice device)
        {
            for (int i = 0; myFrames != null && i < myFrames.Count; i++)
                myFrames[i].EnsureRenderTarget(device);
        }

        #endregion
    }
}
